
/* file HTTaP/src/manager.c
  created sam. avril 25 14:57:58 CEST 2020 by whygee@f-cpu.org
  version 20200429: reworked for 3 return values of HTTaP_server()
  version lun. mai 4 01:20:22 CEST 2020 : H_timediff now accessed by /?ping in HTTaP.c

  implements the management layer of HTTP.
*/

#ifndef H_HTTAP_MANAGER_C
#define H_HTTAP_MANAGER_C
#include "src/manager.h"

// get the (rough) time elapsed since the last successful command:
void get_time_diff() {
  struct timespec current_time;

  clock_gettime(ITIMER_REAL, &current_time);
  H_timediff = timestamp.tv_sec - current_time.tv_sec;
}

// Internal functions that run the show
void change_state_POLL_SLOW () {
  PollState = H_POLL_SLOW;
  H_poll_timeout=0;
  // about 500ms latency to wake up :
  itv_poll.it_value.tv_usec = HTTAP_POLL_SLOW_MS*1000;
  VERBOSE_PRINTF("HTTAP: POLL_SLOW\n");
}

void change_state_POLL_FAST () {
  PollState = H_POLL_FAST;
  H_poll_timeout=0;
  // about 50ms between polls if active :
  itv_poll.it_value.tv_usec = HTTAP_POLL_FAST_MS*1000;
  VERBOSE_PRINTF("HTTAP: POLL_FAST\n");
}

void change_state_BLOCKING () {
//  H_poll_timeout : recomputed before every poll()
  timeout_counter=1;  // only loop once
  PollState = H_BLOCKING;
  VERBOSE_PRINTF("H_BLOCKING\n");
}

void change_state_BLOCKED () {
  H_poll_timeout=-1;  // just tell poll() to wait.
  timeout_counter=1;  // only loop once
  PollState = H_BLOCKED;
  VERBOSE_PRINTF("H_BLOCKED\n");
}

/////////////////////////////////////////////////////////////////

// callback (called when a poll period expired)
void timer_handler(int i) {
  timeout_counter++;
}

/////////////////////// High-level wrappers ///////////////////////

void init_HTTaP(PollState_e s) {
  __sighandler_t e=signal(SIGALRM, timer_handler);
  if ((e!=SIG_IGN) && (e!=SIG_DFL))
    show_exit("\nsignal(): realtime signal is already busy", 1);
#ifdef HTTAP_USE_SIGNAL
  intercept_signals();
#endif
  HTTaP_init();
  bzero(&itv_poll, sizeof(itv_poll));
  clock_gettime(ITIMER_REAL, &timestamp);

  if (s & (H_BLOCKING | H_BLOCKED))
    change_state_BLOCKED ();
  else
    change_state_POLL_SLOW();
}

// These user-visible functions change the mode without interfering with the timeout.
// Can be used within user code, including the HTTaP server itself.

void HTTaP_mode_block() {
  if (PollState == H_POLL_FAST)
    change_state_BLOCKING();
  else
    if (PollState == H_POLL_SLOW)
      change_state_BLOCKED();
}

void HTTaP_mode_poll() {
  if (PollState == H_BLOCKED)
    change_state_POLL_SLOW();
  else
    if (PollState == H_BLOCKING)
      change_state_POLL_FAST();
}

// a wrapper that manages the timeouts :
void HTTaP_manager() {
  int timeout=0;  // flag to send to HTTaP_server()

  timeout_counter=0; // let the work be done by default.

  // Are we waiting for a socket timeout ?
  if (PollState & (H_BLOCKING | H_POLL_FAST) ) {
    get_time_diff();
    // check if the server must close the socket if no input:
    if (H_timediff <= 0)
      timeout = 1;
    else {
      if (PollState == H_BLOCKING)
//      H_poll_timeout = H_timediff * 1000;
        H_poll_timeout = H_timediff << 10; // that's 1024, close enough.
    }
  }
//  else
//    H_timediff = -1;
//  not relevant : if state was H_BLOCKED then it was woken up by accept()
//   so H_timediff can only be requested when (H_BLOCKING | H_POLL_FAST)

  // run the server once:
  switch (HTTaP_server(timeout)) {

    // valid command received: restart the timeout counters !
    case 1:
      clock_gettime(ITIMER_REAL, &timestamp);
      timestamp.tv_sec += keepalive_seconds;

      // re-activate the server ?
      if (PollState & ( H_POLL_SLOW | H_BLOCKED) ) {
        update_httap_session();
        if (PollState == H_POLL_SLOW)
          change_state_POLL_FAST();
        else
          change_state_BLOCKING();
      }
      break;

    // invalid or timeout:
    case 2:
      if (PollState == H_POLL_FAST)
        change_state_POLL_SLOW();
      else
        if (PollState == H_BLOCKING)
          change_state_BLOCKED();
//      break;
  }

  if (PollState & (H_POLL_FAST | H_POLL_SLOW)) {
    // setup the next timeslice.
    if (setitimer(ITIMER_REAL, &itv_poll, NULL) == -1)
      show_exit("\n restarting a timer", 1);
  }
  else
    timeout_counter=1; // just let the host check its own stuff
                       // before halting during the next poll()
}

#endif
