
/* support.c
   (C) 2011,2017,2020 Yann Guidon whygee@f-cpu.org
   The source code of this project is released under the AGPLv3 or later,
   see https://www.gnu.org/licenses/agpl-3.0.en.html

version 20200415: split from HTTaP.h & .c,
  added some portability and simplified some parts
version 20200425: cleanup for the new code structure.
version 20200504 : added itos()

 */


/* convert i to a string (no terminator !)
   and return the number of characters written to *c.
   result would range from 1 to 18 chars (if your int is 64 bits).
   tested OK in the +/-100000000 range. */
int itos(int i, char *c) {
  int size=1; // display at least one character
  unsigned int u, v;

  if (i < 0) {
    *c = '-';
    size++;
    u = -i;  // no overflow should be
     // possible here because the unsigned
     // range is larger than the signed one
  }
  else
    u=i;

  // check the size
  v=u;

  while (v >= 10000) {
    v = v/10000;
    size+=4;
  }

  if (v >= 100) {
    v = v/100;
    size+=2;
  }

  if (v >= 10) {
    size+=1;
  }

  // do the real conversion now:
  c+=size;

  do {
    *(--c) = '0' + (u % 10);
    u = u/10;
  } while (u > 0);

  return size;
}


/* A wrapper to prevent empty environment variables
   from being registered as valid */
void *my_getenv(char *s) {
  s=getenv(s);
  if (s && s[0])
    return s;
  return NULL;
}

/* Display an error message, with optional perror */
void show_exit(char *msg, int perr) {
  fflush(NULL);
  if (perr)
    perror(msg);
  else
    printf(msg);
  exit(EXIT_FAILURE);
}

//////////////// Some signal functions, not used yet ////////////////

#ifdef HTTAP_USE_SIGNAL

// hook for all the signals
void interrupt_program(int sig) {
  printf("\n Received signal %d\n", sig);
  abort_program=1; // raise a flag
}

// Register all the relevant signals to stop the program
void intercept_signals() {
  signal(SIGHUP,  interrupt_program);
  signal(SIGINT,  interrupt_program);
  signal(SIGQUIT, interrupt_program);
  signal(SIGSTOP, interrupt_program);
  signal(SIGTERM, interrupt_program);
  signal(SIGABRT, interrupt_program);
  signal(SIGKILL, interrupt_program);
}

#endif
